@@ -17,19 +17,20 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-import urllib.request, urllib.parse, urllib.error
+import urllib.request
+import urllib.parse
+import urllib.error
 import socket
 import sys
 import re
 import copy
-import time
-import datetime
 
 from Nagstamon.Objects import *
 from Nagstamon.Servers.Generic import GenericServer
 from Nagstamon.Config import conf
 from Nagstamon.Helpers import webbrowser_open
 
+
 class CentreonServer(GenericServer):
     TYPE = 'Centreon'
 
@@ -49,93 +50,104 @@ class CentreonServer(GenericServer):
     # Entries for monitor default actions in context menu
     MENU_ACTIONS = ['Monitor', 'Recheck', 'Acknowledge', 'Downtime']
 
+    # Centreon works better or at all with html.parser for BeautifulSoup
+    PARSER = 'html.parser'
+
     # Needed to detect each Centreon's version
     centreon_version = None
     # Token that centreon use to protect the system
     centreon_token = None
     # To only detect broker once
     first_login = True
+    # limit number of services retrived
+    limit_services_number = 9999
 
     def init_config(self):
         '''
         dummy init_config, called at thread start, not really needed here, just omit extra properties
         '''
-        pass
-
+        # set URLs here already
+        self.init_HTTP()
+        if not self.tls_error:
+            self._define_url()
 
     def init_HTTP(self):
         """
         initialize HTTP connection
         """
-        if self.session == None:
+        if self.session is None:
             GenericServer.init_HTTP(self)
 
-        if self.centreon_version == None:
+        if self.centreon_version is None:
             result_versioncheck = self.FetchURL(self.monitor_cgi_url + '/index.php', giveback='raw')
             raw_versioncheck, error_versioncheck = result_versioncheck.result, result_versioncheck.error
             if error_versioncheck == '':
                 if re.search('2\.2\.[0-9]', raw_versioncheck):
                     self.centreon_version = 2.2
-                    if conf.debug_mode == True:
+                    if conf.debug_mode is True:
                         self.Debug(server=self.get_name(), debug='Centreon version detected : 2.2')
                     # URLs for browser shortlinks/buttons on popup window
-                    self.BROWSER_URLS= { 'monitor': '$MONITOR$/main.php?p=1',\
-                                    'hosts': '$MONITOR$/main.php?p=20103&o=hpb',\
-                                    'services': '$MONITOR$/main.php?p=20202&o=svcpb',\
+                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
+                                    'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
+                                    'services': '$MONITOR$/main.php?p=20202&o=svcpb',
                                     'history': '$MONITOR$/main.php?p=203'}
                 elif re.search('2\.[3-6]\.[0-5]', raw_versioncheck):
                     self.centreon_version = 2.3456
-                    if conf.debug_mode == True:
+                    if conf.debug_mode is True:
                         self.Debug(server=self.get_name(), debug='Centreon version detected : 2.6.5 <=> 2.3')
                     # URLs for browser shortlinks/buttons on popup window
-                    self.BROWSER_URLS= { 'monitor': '$MONITOR$/main.php?p=1',\
-                                    'hosts': '$MONITOR$/main.php?p=20103&o=hpb',\
-                                    'services': '$MONITOR$/main.php?p=20202&o=svcpb',\
+                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
+                                    'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
+                                    'services': '$MONITOR$/main.php?p=20202&o=svcpb',
                                     'history': '$MONITOR$/main.php?p=203'}
                 elif re.search('2\.6\.[6-9]', raw_versioncheck):
                     self.centreon_version = 2.66
-                    if conf.debug_mode == True:
+                    if conf.debug_mode is True:
                         self.Debug(server=self.get_name(), debug='Centreon version detected : 2.6.6')
                     # URLs for browser shortlinks/buttons on popup window
-                    self.BROWSER_URLS= { 'monitor': '$MONITOR$/main.php?p=1',\
-                                    'hosts': '$MONITOR$/main.php?p=20103&o=hpb',\
-                                    'services': '$MONITOR$/main.php?p=20202&o=svcpb',\
+                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
+                                    'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
+                                    'services': '$MONITOR$/main.php?p=20202&o=svcpb',
                                     'history': '$MONITOR$/main.php?p=203'}
                 elif re.search('2\.7\.[0-9]', raw_versioncheck):
                     # Centreon 2.7 only support C. Broker
                     self.centreon_version = 2.7
-                    if conf.debug_mode == True:
+                    if conf.debug_mode is True:
                         self.Debug(server=self.get_name(), debug='Centreon version detected : 2.7')
                     # URLs for browser shortlinks/buttons on popup window
-                    self.BROWSER_URLS= { 'monitor': '$MONITOR$/main.php?',\
-                                    'hosts': '$MONITOR$/main.php?p=20202&o=hpb',\
-                                    'services': '$MONITOR$/main.php?p=20201&o=svcpb',\
+                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
+                                    'hosts': '$MONITOR$/main.php?p=20202&o=hpb',
+                                    'services': '$MONITOR$/main.php?p=20201&o=svcpb',
                                     'history': '$MONITOR$/main.php?p=203'}
                 elif re.search('2\.8\.[0-9]', raw_versioncheck):
                     # Centreon 2.8 only support C. Broker
                     self.centreon_version = 2.8
-                    if conf.debug_mode == True:
+                    if conf.debug_mode is True:
                         self.Debug(server=self.get_name(), debug='Centreon version detected : 2.8')
                     # URLs for browser shortlinks/buttons on popup window
-                    self.BROWSER_URLS= { 'monitor': '$MONITOR$/main.php?',\
-                                    'hosts': '$MONITOR$/main.php?p=20202',\
-                                    'services': '$MONITOR$/main.php?p=20201',\
+                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
+                                    'hosts': '$MONITOR$/main.php?p=20202',
+                                    'services': '$MONITOR$/main.php?p=20201',
                                     'history': '$MONITOR$/main.php?p=203'}
                 else:
                     # unsupported version or unable do determine
                     self.centreon_version = 2.8
-                    if conf.debug_mode == True:
+                    if conf.debug_mode is True:
                         self.Debug(server=self.get_name(), debug='Centreon version unknown : supposed to be >= 2.8')
                     # URLs for browser shortlinks/buttons on popup window
-                    self.BROWSER_URLS= { 'monitor': '$MONITOR$/main.php?',\
-                                    'hosts': '$MONITOR$/main.php?p=20202&o=hpb',\
-                                    'services': '$MONITOR$/main.php?p=20201&o=svcpb',\
+                    self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
+                                    'hosts': '$MONITOR$/main.php?p=20202&o=hpb',
+                                    'services': '$MONITOR$/main.php?p=20201&o=svcpb',
                                     'history': '$MONITOR$/main.php?p=203'}
             else:
-                if conf.debug_mode == True:
+                if conf.debug_mode is True:
                     self.Debug(server=self.get_name(), debug='Error getting the home page : ' + error_versioncheck)
-            del result_versioncheck, raw_versioncheck, error_versioncheck
 
+            if self.first_login:
+                self.SID = self._get_sid().result
+                self.first_login = False
+
+            del result_versioncheck, raw_versioncheck, error_versioncheck
 
     def reset_HTTP(self):
         '''
@@ -144,19 +156,18 @@ class CentreonServer(GenericServer):
         self.SID = None
         self.SID = self._get_sid().result
 
-
     def open_monitor(self, host, service=''):
-        if self.use_autologin == True:
+        if self.use_autologin is True:
             auth = '&autologin=1&useralias=' + self.username + '&token=' + self.autologin_key
         else:
             auth = ''
 
         #  Meta - Centreon < 2.7
         if host == '_Module_Meta' and self.centreon_version < 2.7:
-            webbrowser_open(self.urls_centreon['index'] + '?' + urllib.parse.urlencode({'p':20206,'o':'meta'}) + auth )
+            webbrowser_open(self.urls_centreon['index'] + '?' + urllib.parse.urlencode({'p': 20206, 'o': 'meta'}) + auth )
         #  Meta - Centreon 2.7
         elif host == '_Module_Meta' and self.centreon_version == 2.7:
-            webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':20206,'o':'meta'}) + auth )
+            webbrowser_open(self.urls_centreon['main'] + '?' + urllib.parse.urlencode({'p':20206, 'o':'meta'}) + auth )
         #  Meta - Centreon 2.8
         elif host == '_Module_Meta' and self.centreon_version == 2.8:
             m =  re.search(r'^.+ \((?P<rsd>.+)\)$', service)
@@ -220,7 +231,6 @@ class CentreonServer(GenericServer):
             # those broker urls would not be changing too often so this check migth be done here
             if self.first_login:
                 self._get_xml_path(sid)
-                self._define_url()
                 self.first_login = False
             return Result(result=sid)
 
@@ -494,18 +504,18 @@ class CentreonServer(GenericServer):
 
         # services (unknown, warning or critical?)
         if self.centreon_version == 2.7 or self.centreon_version == 2.8:
-            nagcgiurl_services = self.urls_centreon['xml_services'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'svcpb', 'p':20201, 'nc':0, 'criticality':0, 'statusService':'svcpb', 'sSetOrderInMemory':1, 'sid':self.SID})
+            nagcgiurl_services = self.urls_centreon['xml_services'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'svcpb', 'p':20201, 'nc':0, 'criticality':0, 'statusService':'svcpb', 'sSetOrderInMemory':1, 'sid':self.SID})
         else:
-            nagcgiurl_services = self.urls_centreon['xml_services'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'svcpb', 'sort_type':'status', 'sid':self.SID})
+            nagcgiurl_services = self.urls_centreon['xml_services'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'svcpb', 'sort_type':'status', 'sid':self.SID})
 
         # hosts (up or down or unreachable)
         # define hosts xml URL, because of inconsistant url
         if self.centreon_version == 2.7:
-            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'hpb', 'p':20202, 'criticality':0, 'statusHost':'hpb', 'sSetOrderInMemory':1, 'sid':self.SID})
+            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'hpb', 'p':20202, 'criticality':0, 'statusHost':'hpb', 'sSetOrderInMemory':1, 'sid':self.SID})
         elif self.centreon_version == 2.8:
-            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'hpb', 'p':20202, 'criticality':0, 'statusHost':'hpb', 'sSetOrderInMemory':1, 'sid':self.SID})
+            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'hpb', 'p':20202, 'criticality':0, 'statusHost':'hpb', 'sSetOrderInMemory':1, 'sid':self.SID})
         else:
-            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'hpb', 'sort_type':'status', 'sid':self.SID})
+            nagcgiurl_hosts = self.urls_centreon['xml_hosts'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'hpb', 'sort_type':'status', 'sid':self.SID})
 
         # hosts - mostly the down ones
         # unfortunately the hosts status page has a different structure so
@@ -516,10 +526,17 @@ class CentreonServer(GenericServer):
 
             # check if any error occured
             errors_occured = self.check_for_error(xmlobj, error, status_code)
+
             # if there are errors return them
             if errors_occured != False:
                 return(errors_occured)
 
+            # Check if the result is not empty
+            if len(xmlobj) == 0:
+                if conf.debug_mode == True:
+                    self.Debug(server=self.get_name(), debug='Empty host XML result')
+                return Result(result=None, error="Empty host XML result")
+
             # in case there are no children session ID is expired
             if xmlobj.text.lower() == 'bad session id':
                 del xmlobj
@@ -530,10 +547,10 @@ class CentreonServer(GenericServer):
                 self.SID = self._get_sid().result
                 result = self.FetchURL(nagcgiurl_hosts, giveback='xml')
                 xmlobj, error, status_code = result.result, result.error, result.status_code
-                if error != '' or status_code > 400:
-                    return Result(result=copy.deepcopy(xmlobj),
-                                  error=copy.deepcopy(error),
-                                  status_code=status_code)
+                errors_occured = self.check_for_error(xmlobj, error, status_code)
+                # if there are errors return them
+                if errors_occured != False:
+                    return(errors_occured)
 
                 # a second time a bad session id should raise an error
                 if xmlobj.text.lower() == 'bad session id':
@@ -600,6 +617,12 @@ class CentreonServer(GenericServer):
             if errors_occured != False:
                 return(errors_occured)
 
+            # Check if the result is not empty
+            if len(xmlobj) == 0:
+                if conf.debug_mode == True:
+                    self.Debug(server=self.get_name(), debug='Empty service XML result')
+                return Result(result=None, error="Empty service XML result")
+
             # in case there are no children session id is invalid
             if xmlobj.text.lower() == 'bad session id':
                 # debug
@@ -609,10 +632,11 @@ class CentreonServer(GenericServer):
                 self.SID = self._get_sid().result
                 result = self.FetchURL(nagcgiurl_services, giveback='xml')
                 xmlobj, error, status_code = result.result, result.error, result.status_code
-                if error != '' or status_code > 400:
-                    return Result(result=copy.deepcopy(xmlobj),
-                                  error=copy.deepcopy(error),
-                                  status_code=status_code)
+                errors_occured = self.check_for_error(xmlobj, error, status_code)
+                # if there are errors return them
+                if errors_occured != False:
+                    return(errors_occured)
+
                 # a second time a bad session id should raise an error
                 if xmlobj.text.lower() == 'bad session id':
                     return Result(result='ERROR',
@@ -623,17 +647,20 @@ class CentreonServer(GenericServer):
             if self.centreon_version < 2.8:
                 # define meta-services xml URL
                 if self.centreon_version == 2.7:
-                    nagcgiurl_meta_services = self.urls_centreon['xml_meta'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'meta', 'sort_type':'status', 'sid':self.SID})
+                    nagcgiurl_meta_services = self.urls_centreon['xml_meta'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'meta', 'sort_type':'status', 'sid':self.SID})
                 else:
-                    nagcgiurl_meta_services = self.urls_centreon['xml_meta'] + '?' + urllib.parse.urlencode({'num':0, 'limit':999, 'o':'meta', 'sort_type':'status', 'sid':self.SID})
+                    nagcgiurl_meta_services = self.urls_centreon['xml_meta'] + '?' + urllib.parse.urlencode({'num':0, 'limit':self.limit_services_number, 'o':'meta', 'sort_type':'status', 'sid':self.SID})
 
                 # retrive meta-services xml STATUS
                 result_meta = self.FetchURL(nagcgiurl_meta_services, giveback='xml')
                 xmlobj_meta, error_meta, status_code_meta = result_meta.result, result_meta.error, result_meta.status_code
-                if error_meta != '' or status_code_meta > 400:
-                    return Result(result=xmlobj_meta,
-                                  error=copy.deepcopy(error_meta),
-                                  status_code=status_code_meta)
+
+                # check if any error occured
+                errors_occured = self.check_for_error(xmlobj_meta, error_meta, status_code_meta)
+
+                # if there are errors return them
+                if errors_occured != False:
+                    return(errors_occured)
 
                 # a second time a bad session id should raise an error
                 if xmlobj_meta.text.lower() == 'bad session id':
@@ -687,18 +714,19 @@ class CentreonServer(GenericServer):
                             self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type = str(l.ca.text).split(' ')
 
                         # disgusting workaround for https://github.com/HenriWahl/Nagstamon/issues/91
-                        # pretty sure it's only 2.5, but we don't have special code for 2.5
-                        if self.centreon_version < 2.66:
-                            if self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status in self.TRANSLATIONS:
-                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status = self.TRANSLATIONS[\
-                                self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status]
+                        # Still needed in Centreon 2.8 at least : https://github.com/HenriWahl/Nagstamon/issues/344
+                        # Need enhancement, we can do service state matching with this field <sc>service_unknown</sc>
+                        #if self.centreon_version < 2.66:
+                        if self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status in self.TRANSLATIONS:
+                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status = self.TRANSLATIONS[\
+                            self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status]
 
                         if not (self.centreon_version < 2.8 and self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].host == '_Module_Meta'):
                             self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type =\
                             self.HARD_SOFT[self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type]
 
                         if conf.debug_mode == True:
-                            self.Debug(server=self.get_name(), debug='Service status type : ' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].name + '/' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type)
+                            self.Debug(server=self.get_name(), debug='Service / status_type : ' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].name + '/' + self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_type)
                         self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].last_check = str(l.lc.text)
                         self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].duration = str(l.d.text)
                         self.new_hosts[str(l.hn.text)].services[str(l.sd.text)].status_information = str(l.po.text).replace('\n', ' ').strip()
@@ -932,7 +960,11 @@ class CentreonServer(GenericServer):
         except:
             self.Error(sys.exc_info())
 
+    # This Hook seems to not be called anymore from main loop
     def Hook(self):
+        # debug
+        if conf.debug_mode == True:
+            self.Debug(server=self.get_name(), debug='Hook function')
         '''
         in case count is down get a new SID, just in case
         was kicked out but as to be seen in https://sourceforge.net/p/nagstamon/bugs/86/ there are problems with older
@@ -953,3 +985,4 @@ class CentreonServer(GenericServer):
             self.SIDcount = 0
         else:
             self.SIDcount += 1
+
