@@ -7,7 +7,7 @@ import urllib.request
 import urllib.parse
 import urllib.error
 import time
-import socket
+# import socket  # never used
 
 from Nagstamon.Helpers import (HumanReadableDurationFromTimestamp,
                                webbrowser_open)
@@ -17,7 +17,9 @@ from Nagstamon.Objects import (GenericHost,
                                Result)
 from Nagstamon.Servers.Generic import GenericServer
 from Nagstamon.thirdparty.zabbix_api import (ZabbixAPI,
-                                             ZabbixAPIException)
+                                             ZabbixAPIException,
+                                             APITimeout,
+                                             Already_Exists)
 
 
 class ZabbixError(Exception):
@@ -48,43 +50,40 @@ class ZabbixServer(GenericServer):
             'UNKN': 'UNKNOWN',
             'PEND': 'PENDING',
             '0': 'OK',
-            '1': 'UNKNOWN',
+            '1': 'INFORMATION',
             '2': 'WARNING',
-            '5': 'CRITICAL',
-            '3': 'WARNING',
-            '4': 'CRITICAL'}
+            '3': 'AVERAGE',
+            '4': 'HIGH',
+            '5': 'DISASTER'}
 
         # Entries for monitor default actions in context menu
         self.MENU_ACTIONS = ["Recheck", "Acknowledge", "Downtime"]
         self.username = conf.servers[self.get_name()].username
         self.password = conf.servers[self.get_name()].password
+        self.ignore_cert = conf.servers[self.get_name()].ignore_cert
+        if self.ignore_cert is True:
+            self.validate_certs = False
+        else:
+            self.validate_certs = True
 
     def _login(self):
         try:
-            self.zapi = ZabbixAPI(server=self.monitor_url, path="", log_level=0)
+            self.zapi = ZabbixAPI(server=self.monitor_url, path="", log_level=0,
+                                  validate_certs=self.validate_certs)
             self.zapi.login(self.username, self.password)
         except ZabbixAPIException:
             result, error = self.Error(sys.exc_info())
             return Result(result=result, error=error)
 
-    """
-    def init_HTTP(self):
-
-        # not necessary to be initialized wit hevery HTTP request
-        self.statemap = {
-            'UNREACH': 'UNREACHABLE',
-            'CRIT': 'CRITICAL',
-            'WARN': 'WARNING',
-            'UNKN': 'UNKNOWN',
-            'PEND': 'PENDING',
-            '0': 'OK',
-            '1': 'UNKNOWN',
-            '2': 'WARNING',
-            '5': 'CRITICAL',
-            '3': 'WARNING',
-            '4': 'CRITICAL'}
-        GenericServer.init_HTTP(self)
-    """
+    def getLastApp(self, this_item):
+        if len(this_item) > 0:
+            last_app = len(this_item[0]['applications']) - 1  # use it to get the last application name
+            if last_app > -1:
+                return "%s" % this_item[0]['applications'][last_app]['name']
+            else:
+                return "NO APP"
+        else:
+            return "Web scenario"
 
     def _get_status(self):
         """
@@ -105,7 +104,7 @@ class ZabbixServer(GenericServer):
             try:
                 hosts = self.zapi.host.get(
                     {"output": ["host", "ip", "status", "available", "error", "errors_from"], "filter": {}})
-            except (ZabbixError, ZabbixAPIException):
+            except (ZabbixError, ZabbixAPIException, APITimeout, Already_Exists):
                 # set checking flag back to False
                 self.isChecking = False
                 result, error = self.Error(sys.exc_info())
@@ -174,8 +173,8 @@ class ZabbixServer(GenericServer):
                     if int(x['internal']) == 0]
 
                 zabbix_triggers = self.zapi.trigger.get(
-                    {'sortfield': 'lastchange', 'withLastEventUnacknowledged': True, 'groupids': hostgroup_ids,
-                     "monitored": True, "filter": {'value': 1}})
+                    {'sortfield': 'lastchange', 'skipDependent': True, 'withLastEventUnacknowledged': True, 'groupids': hostgroup_ids,
+                     'monitored': True, 'filter': {'value': 1}})
 
                 triggers_list = []
 
@@ -187,27 +186,30 @@ class ZabbixServer(GenericServer):
                      'output': 'extend',
                      'select_items': 'extend',  # thats for zabbix api 1.8
                      'selectItems': 'extend',  # thats for zabbix api 2.0+
-                     'expandData': True}
+                     'expandData': True,
+                     'selectHosts': 'extend'}
                 )
                 if type(this_trigger) is dict:
                     for triggerid in list(this_trigger.keys()):
                         services.append(this_trigger[triggerid])
+                        # get Application name for the trigger
                         this_item = self.zapi.item.get(
                             {'itemids': [this_trigger[triggerid]['items'][0]['itemid']],
                              'selectApplications': 'extend'}
                         )
-                        last_app = len(this_item[0]['applications']) - 1
-                        this_trigger[triggerid]['application'] = this_item[0]['applications'][last_app]['name']
+
+                        this_trigger[triggerid]['application'] = self.getLastApp(this_item)
+
                 elif type(this_trigger) is list:
                     for trigger in this_trigger:
                         services.append(trigger)
+                        # get Application name for the trigger
                         this_item = self.zapi.item.get(
                             {'itemids': trigger['items'][0]['itemid'],
                              'selectApplications': 'extend'}
                         )
-                        # last_app = 0  # use it to get the first application name
-                        last_app = len(this_item[0]['applications']) - 1  # use it to get the last application name
-                        trigger['application'] = this_item[0]['applications'][last_app]['name']
+
+                        trigger['application'] = self.getLastApp(this_item)
 
             except ZabbixAPIException:
                 # FIXME Is there a cleaner way to handle this? I just borrowed
@@ -230,6 +232,8 @@ class ZabbixServer(GenericServer):
                 # UPDATE Zabbix api 3.0 doesn't but I didn't tried with older
                 #        so I left it
                 status = self.statemap.get(service['priority'], service['priority'])
+                # self.Debug(server=self.get_name(), debug="SERVICE (" + service['application'] + ") STATUS: **" + status + "** PRIORITY: #" + service['priority'])
+                # self.Debug(server=self.get_name(), debug="-----======== SERVICE " + str(service))
                 if not status == 'OK':
                     if not service['description'].endswith('...'):
                         state = service['description']
@@ -258,9 +262,8 @@ class ZabbixServer(GenericServer):
                         'command': 'zabbix',
                         'triggerid': service['triggerid'],
                     }
-
                     if api_version >= '3.0':
-                        n['host'] = self.zapi.host.get({"output": ["host"], "filter": {}, "triggerids": service['triggerid']})[0]['host']
+                        n['host'] = service['hosts'][0]['host']
                     else:
                         n['host'] = service['host']
 
@@ -280,8 +283,10 @@ class ZabbixServer(GenericServer):
                             self.new_hosts[n["host"]].status = "DOWN"
                             # also take duration from "service" aka trigger
                             self.new_hosts[n["host"]].duration = n["duration"]
+                            if conf.debug_mode is True:
+                                self.Debug(server=self.get_name(), debug="Adding Host[" + n['host'] + "]")
                         else:
-                            new_service = n["service"]
+                            new_service = n["triggerid"]
                             self.new_hosts[n["host"]].services[new_service] = GenericService()
                             self.new_hosts[n["host"]].services[new_service].host = n["host"]
                             self.new_hosts[n["host"]].services[new_service].name = n["service"]
@@ -296,6 +301,8 @@ class ZabbixServer(GenericServer):
                             self.new_hosts[n["host"]].services[new_service].address = n["host"]
                             self.new_hosts[n["host"]].services[new_service].command = n["command"]
                             self.new_hosts[n["host"]].services[new_service].triggerid = n["triggerid"]
+                            if conf.debug_mode is True:
+                                self.Debug(server=self.get_name(), debug="Adding new service[" + new_service + "] **" + n['service'] + "**")
 
         except (ZabbixError, ZabbixAPIException):
             # set checking flag back to False
@@ -335,36 +342,36 @@ class ZabbixServer(GenericServer):
                        debug="Open host/service monitor web page " + url)
         webbrowser_open(url)
 
-    def GetHost(self, host):
-        """
-            find out ip or hostname of given host to access hosts/devices which do not appear in DNS but
-            have their ip saved in Nagios
-        """
+    # def GetHost(self, host):
+    #     """
+    #         find out ip or hostname of given host to access hosts/devices which do not appear in DNS but
+    #         have their ip saved in Nagios
+    #     """
 
-        # the fasted method is taking hostname as used in monitor
-        if conf.connect_by_host is True:
-            return Result(result=host)
+    #     # the fasted method is taking hostname as used in monitor
+    #     if conf.connect_by_host is True:
+    #         return Result(result=host)
 
-        ip = ""
+    #     ip = ""
 
-        try:
-            if host in self.hosts:
-                ip = self.hosts[host].address
-            if conf.debug_mode is True:
-                self.Debug(server=self.get_name(), host=host, debug="IP of %s:" % host + " " + ip)
+    #     try:
+    #         if host in self.hosts:
+    #             ip = self.hosts[host].address
+    #         if conf.debug_mode is True:
+    #             self.Debug(server=self.get_name(), host=host, debug="IP of %s:" % host + " " + ip)
 
-            if conf.connect_by_dns is True:
-                try:
-                    address = socket.gethostbyaddr(ip)[0]
-                except socket.herror:
-                    address = ip
-            else:
-                address = ip
-        except ZabbixError:
-            result, error = self.Error(sys.exc_info())
-            return Result(result=result, error=error)
+    #         if conf.connect_by_dns is True:
+    #             try:
+    #                 address = socket.gethostbyaddr(ip)[0]
+    #             except socket.herror:
+    #                 address = ip
+    #         else:
+    #             address = ip
+    #     except ZabbixError:
+    #         result, error = self.Error(sys.exc_info())
+    #         return Result(result=result, error=error)
 
-        return Result(result=address)
+        # return Result(result=address)
 
     def _set_recheck(self, host, service):
         pass
@@ -383,9 +390,15 @@ class ZabbixServer(GenericServer):
             self._login()
         events = []
         for e in self.zapi.event.get({'triggerids': params['triggerids'],
-                                      'hide_unknown': True,
+                                      # from zabbix 2.2 should be used "objectids" instead of "triggerids"
+                                      'objectids': params['triggerids'],
+                                      'hide_unknown': True,  # zabbix 1.8
+                                      'acknowledged': False,
                                       'sortfield': 'clock',
-                                      'sortorder': 'desc'}):
+                                      'sortorder': 'DESC'}):
+            # stop at first event in "OK" status
+            if e['value'] == '0':
+                break
             events.append(e['eventid'])
         self.zapi.event.acknowledge({'eventids': events, 'message': params['message']})
 
